import os
from contextlib import asynccontextmanager

import numpy as np
import tensorflow as tf
from fastapi import FastAPI, HTTPException
from huggingface_hub import from_pretrained_keras

models = {}
classes = ['dandelion', 'daisy', 'tulip', 'sunflower', 'rose']
image_size = 256


def search_file(flower: str):
    files = [f for f in os.listdir("./images") if flower in f]
    if len(files) == 0:
        raise HTTPException(status_code=404, detail="Image not found")
    if len(files) > 1:
        raise HTTPException(status_code=500, detail="The image you requested was found multiple times")
    return files[0]


@asynccontextmanager
async def lifespan(app: FastAPI):
    models['mobile-vit'] = from_pretrained_keras("keras-io/mobile-vit-xxs")
    yield
    models.clear()


app = FastAPI(lifespan=lifespan)


@app.get("/predict/{flower}", tags=['Classification App'])
async def predict(flower: str):
    filename = search_file(flower)
    image = tf.io.read_file(f"./images/{filename}")
    image = tf.image.decode_image(image, channels=3)
    image = tf.image.resize(image, (image_size, image_size))
    image = tf.expand_dims(image, axis=0)
    prediction = models['mobile-vit'].predict(image)
    print("Prediction: {}".format(prediction))
    prediction = tf.squeeze(tf.round(prediction))
    print("Squeezed prediction: {}".format(prediction))
    text_output = str(f'{classes[(np.argmax(prediction))]}')
    return {"prediction": text_output}
