from datetime import date as date_type, datetime, time as time_type

from fastapi import FastAPI, Request
from pydantic import BaseSettings

from source.dataset_service import get_acquired_ndmi_by_date, get_predicted_ndmi_by_date, get_cumulated_pluvio_by_date, \
    get_predicted_flood_by_date, get_predicted_flood_by_day
from source.resources import AcquiredNdmiResource, PredictedNdmiResource, CumulatedPluvioResource, \
    PredictedFloodResource


class Settings(BaseSettings):
    hf_token: str


app = FastAPI()
settings = Settings()


@app.get("/acquired-ndmis", tags=['EO4NOWCAST'])
async def get_acquired_ndmis(date: date_type, request: Request) -> AcquiredNdmiResource:
    return get_acquired_ndmi_by_date(date, settings.hf_token)


@app.get("/predicted-ndmis", tags=['EO4NOWCAST'])
async def get_predicted_ndmis(date: date_type, time: time_type, request: Request) -> PredictedNdmiResource:
    host = request.headers.get('host')
    return get_predicted_ndmi_by_date(date, time, settings.hf_token, host)


@app.get("/cumulated-pluvio", tags=['EO4NOWCAST'])
async def get_cumulated_pluvio(date: date_type, request: Request) -> CumulatedPluvioResource:
    host = request.headers.get('host')
    return get_cumulated_pluvio_by_date(date, settings.hf_token, host)


@app.get("/predicted-flood", tags=['EO4NOWCAST'])
async def get_predicted_flood(date: datetime, request: Request, station: str | None = None)\
        -> list[PredictedFloodResource]:
    host = request.headers.get('host')
    return get_predicted_flood_by_date(date, station, settings.hf_token, host)


@app.get("/predicted-flood-day", tags=['EO4NOWCAST'])
async def get_predicted_flood_day(date: date_type, request: Request, station: str | None = None)\
        -> list[PredictedFloodResource]:
    host = request.headers.get('host')
    return get_predicted_flood_by_day(date, station, settings.hf_token, host)
